# _*_ coding: utf-8 _*_
#
# hspylib v0.11.200
#
# Package: main.hspylib.modules.application.argparse
"""Package initialization."""

__all__ = [
    'argument_parser', 
    'arguments_builder', 
    'chained_arguments_builder', 
    'options_builder', 
    'parser_action'
]
__version__ = '0.11.200'
